package javax.swing;

import java.awt.Component;
import java.awt.Container;
import java.awt.FocusTraversalPolicy;

public class DefaultFocusManager extends FocusManager
{
    final FocusTraversalPolicy gluePolicy;
    private final FocusTraversalPolicy layoutPolicy;
    private final LayoutComparator comparator;
    
    public DefaultFocusManager() {
        this.gluePolicy = new LegacyGlueFocusTraversalPolicy(this);
        this.layoutPolicy = new LegacyLayoutFocusTraversalPolicy(this);
        this.comparator = new LayoutComparator();
        this.setDefaultFocusTraversalPolicy(this.gluePolicy);
    }
    
    public Component getComponentAfter(Container container, Component component) {
        Container container2 = container.isFocusCycleRoot() ? container : container.getFocusCycleRootAncestor();
        if (container2 != null) {
            FocusTraversalPolicy focusTraversalPolicy = container2.getFocusTraversalPolicy();
            if (focusTraversalPolicy != this.gluePolicy) {
                return focusTraversalPolicy.getComponentAfter(container2, component);
            }
            this.comparator.setComponentOrientation(container2.getComponentOrientation());
            return this.layoutPolicy.getComponentAfter(container2, component);
        }
        else {
            return null;
        }
    }
    
    public Component getComponentBefore(Container container, Component component) {
        Container container2 = container.isFocusCycleRoot() ? container : container.getFocusCycleRootAncestor();
        if (container2 != null) {
            FocusTraversalPolicy focusTraversalPolicy = container2.getFocusTraversalPolicy();
            if (focusTraversalPolicy != this.gluePolicy) {
                return focusTraversalPolicy.getComponentBefore(container2, component);
            }
            this.comparator.setComponentOrientation(container2.getComponentOrientation());
            return this.layoutPolicy.getComponentBefore(container2, component);
        }
        else {
            return null;
        }
    }
    
    public Component getFirstComponent(Container container) {
        Container container2 = container.isFocusCycleRoot() ? container : container.getFocusCycleRootAncestor();
        if (container2 != null) {
            FocusTraversalPolicy focusTraversalPolicy = container2.getFocusTraversalPolicy();
            if (focusTraversalPolicy != this.gluePolicy) {
                return focusTraversalPolicy.getFirstComponent(container2);
            }
            this.comparator.setComponentOrientation(container2.getComponentOrientation());
            return this.layoutPolicy.getFirstComponent(container2);
        }
        else {
            return null;
        }
    }
    
    public Component getLastComponent(Container container) {
        Container container2 = container.isFocusCycleRoot() ? container : container.getFocusCycleRootAncestor();
        if (container2 != null) {
            FocusTraversalPolicy focusTraversalPolicy = container2.getFocusTraversalPolicy();
            if (focusTraversalPolicy != this.gluePolicy) {
                return focusTraversalPolicy.getLastComponent(container2);
            }
            this.comparator.setComponentOrientation(container2.getComponentOrientation());
            return this.layoutPolicy.getLastComponent(container2);
        }
        else {
            return null;
        }
    }
    
    public boolean compareTabOrder(Component component, Component component2) {
        return this.comparator.compare(component, component2) < 0;
    }
}
